+ /*
+  * Copyright (c) 1983, 1993
+  *	The Regents of the University of California.  All rights reserved.
+  *
+  * Redistribution and use in source and binary forms, with or without
+  * modification, are permitted provided that the following conditions
+  * are met:
+  * 1. Redistributions of source code must retain the above copyright
+  *    notice, this list of conditions and the following disclaimer.
+  * 2. Redistributions in binary form must reproduce the above copyright
+  *    notice, this list of conditions and the following disclaimer in the
+  *    documentation and/or other materials provided with the distribution.
+  * 3. All advertising materials mentioning features or use of this software
+  *    must display the following acknowledgement:
+  *	This product includes software developed by the University of
+  *	California, Berkeley and its contributors.
+  * 4. Neither the name of the University nor the names of its contributors
+  *    may be used to endorse or promote products derived from this software
+  *    without specific prior written permission.
+  *
+  * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
+  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+  * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
+  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+  * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+  * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+  * SUCH DAMAGE.
+  */
+ 
+ /*
+  * Talk: This adaptation of 'talk' is derived directly from the version
+  * in the 4.4BSD UNIX OS.  It is an attempt to make a talk client that is 
+  * indifferent to the number and state of talk sessions currently in progress.
+  * It was intended that this be used in the IRC II/EPIC clients and I do
+  * not make any guarantee that it will be easily adapted to any other use,
+  * although it is my hope that it is.  No functionality has been knowingly
+  * changed save for the obvious changes neccesary to integrate it into
+  * the ircII client.
+  *
+  * XXXX - This file is a mess.  But it works.  I need to clean it up 
+  * though, because if anything were to break, im not sure if id be able 
+  * to find it. >;-)
+  */
+ 
+ #include <sys/types.h>
+ #include <sys/param.h>
+ #include <sys/socket.h>
+ #include <sys/ioctl.h>
+ #include <sys/time.h>
+ #include <netinet/in.h>
+ #include "talkd.h"
+ #include <pwd.h>
+ #include <stdio.h>
+ #include <errno.h>
+ #include <string.h>
+ #include <netdb.h>
+ 
+ #include "talk.h"
+ 
+ #include "irc.h"
+ #include "ircaux.h"
+ #include "hook.h"
+ #include "timer.h"
+ #include "output.h"
+ 
+ #ifndef TALK_SUPPORT
+ #ifdef __STDC__
+ void talkcmd (char *cmd, char *args, char *subargs)
+ #else
+ void talkcmd (cmd, args, subargs)
+ char *cmd, *args, *subargs;
+ #endif
+ {
+ 	yell("Sorry, this client does not have TALK support");
+ }
+ #else
+ 
+ 
+ static	char *answers[] = {
+ 	"answer #0",					/* SUCCESS */
+ 	"Your party is not logged on",			/* NOT_HERE */
+ 	"Target machine is too confused to talk to us",	/* FAILED */
+ 	"Target machine does not recognize us",		/* MACHINE_UNKNOWN */
+ 	"Your party is refusing messages",		/* PERMISSION_REFUSED */
+ 	"Target machine can not handle remote talk",	/* UNKNOWN_REQUEST */
+ 	"Target machine indicates protocol mismatch",	/* BADVERSION */
+ 	"Target machine indicates protocol botch (addr)",/* BADADDR */
+ 	"Target machine indicates protocol botch (ctl_addr)",/* BADCTLADDR */
+ };
+ #define	NANSWERS	(sizeof (answers) / sizeof (answers[0]))
+ 
+ static int get_names _((TalkSession *talker, char *remote_peer));
+ static int open_ctl _((TalkSession *talker));
+ static int open_sockt _((TalkSession *talker));
+ static int check_for_invitation _((TalkSession *talker));
+ static int invite_remote _((TalkSession *talker));
+ static int set_edit_chars _((TalkSession *talker));
+ static int ctl_transact _((TalkSession *talker, struct in_addr *target, int type, CTL_RESPONSE *rp));
+ static TalkSession *get_new_talk _((void));
+ static int delete_talk _((TalkSession **talker, char *message));
+ static int send_delete _((TalkSession *talker));
+ static void setup_local_msg _((TalkSession *talker, long id));
+ static int leave_invitation _((TalkSession *talker));
+ static int send_talk_announce _((void *arg));
+ static int establish_waiting_talk_connection _((TalkSession *talker));
+ static TalkSession *lookup_talk _((char *userhost));
+ static void list_talk _((void));
+ 
+ #ifdef __STDC__
+ void talkcmd (char *cmd, char *args, char *subargs)
+ #else
+ void talkcmd (cmd, args, subargs)
+ char *cmd, *args, *subargs;
+ #endif
+ {
+ 	char *userhost;
+ 	char *argument;
+ 
+ 	if ((*args == '-') || (*args == '/'))
+ 	{
+ 		args++;
+ 		argument = next_arg(args, &args);
+ 		if (!my_strnicmp(argument, "D", 1))
+ 		{
+ 			char *uh = next_arg(args, &args);
+ 			TalkSession *talkptr;
+ 			if (!uh)
+ 				return;
+ 			talkptr = lookup_talk(uh);
+ 			if (!talkptr)
+ 			{
+ 				yell("You do not have a talk open to (%s)", uh);
+ 				return;
+ 			}
+ 			delete_talk(&talkptr, "You requested it.");
+ 		}
+ 	}
+ 
+ 	else if ((userhost = next_arg(args, &args)))
+ 	{
+ 		if (!do_talk(userhost))
+ 			yell("Looks ok...");
+ 		else
+ 			yell("Some sort of error...");
+ 	}
+ 
+ 	else
+ 		list_talk();
+ }
+ 
+ /*
+  * This is it.  When you have decided who it is you want to talk to, you
+  * call this function and it handles the whole thing.  It returns -1 if
+  * the connection was aborted, and it returns 0 if the connection was
+  * successfully established or if the connection is still pending (if we
+  * had to send an invitiation which may take a long time to be answered)
+  */
+ #ifdef __STDC__
+ int do_talk (char *remote_peer)
+ #else
+ int do_talk(remote_peer)
+ char *remote_peer;
+ #endif
+ {
+ 	TalkSession *talker = get_new_talk();
+ 
+ 	/*
+ 	 * Save the name the user wants to use
+ 	 */
+ 	talker->user_ref = m_strdup(remote_peer);
+ 
+ 	/*
+ 	 * resolve the names of both my machine and the peer's machine,
+ 	 * doing any checking for usernames and whatnot
+ 	 */
+ 	if (get_names(talker, remote_peer) == -1)
+ 		return -1;
+ 
+ 	/*
+ 	 * Set up a passive UDP socket, used for communciations
+ 	 * with the two talkd's
+ 	 */
+ 	if (open_ctl(talker) == -1)
+ 		return -1;
+ 
+ 	/*
+ 	 * Set up a TCP socket as a listening socket, because we assume
+ 	 * that we have not been invited.  If it ends up we have been
+ 	 * invited, then we go ahead and use it as an active connection.
+ 	 */
+ 	if (open_sockt(talker) == -1)
+ 		return -1;
+ 
+ 	/*
+ 	 * If check_local returns nonzero, that means there were no 
+ 	 * invitations waiting for us on either machine
+ 	 */
+ 	switch (check_for_invitation(talker))
+ 	{
+ 		case -1:			/* error */
+ 			return -1;
+ 		case 1:				/* no invitation */
+ 			return invite_remote(talker);
+ 		default:			/* invited, now connected */
+ 			break;
+ 	}
+ 
+ 
+ 	/*
+ 	 * Ok. if we have a connection established, then we go ahead
+ 	 * and exchange editing control characters with them.
+ 	 */
+ 	if (set_edit_chars(talker) == -1)
+ 		return -1;
+ 
+ 	return 0;
+ }
+ 
+ 
+ /*
+  * Determine the local and remote user, tty, and machines, and set up a
+  * talk session struct
+  */
+ #ifdef __STDC__
+ static int get_names(TalkSession *talker, char *remote_peer)
+ #else
+ static int get_names (talker, remote_peer)
+ TalkSession *talker;
+ char *remote_peer;
+ #endif
+ {
+ 	char *cp, *names;
+ 	struct passwd *pw;
+ 	struct hostent *hp;
+ 	struct servent *sp;
+ 
+ 	if ((cp = getlogin()) == NULL)
+ 	{
+ 		if ((pw = getpwuid(getuid())) == NULL) 
+ 			return delete_talk(&talker, "Sorry, cant figure out your username.");
+ 		strmcpy(talker->local_user, pw->pw_name, MAXLOGNAME-1);
+ 	}
+ 	else
+ 		strmcpy(talker->local_user, cp, MAXLOGNAME-1);
+ 
+ 	gethostname(talker->local_host, MAXHOSTNAMELEN);
+ 
+ 	/* check for, and strip out, the machine name of the target */
+ 	names = m_strdup(remote_peer);
+ 	for (cp = names; *cp && !index("@:!.", *cp); cp++)
+ 		;
+ 	if (*cp == '\0') 
+ 	{
+ 		/* this is a local to local talk */
+ 		strmcpy(talker->remote_user, names, MAXLOGNAME-1);
+ 		strmcpy(talker->remote_host, talker->local_host, MAXHOSTNAMELEN-1);
+ 	} 
+ 	else 
+ 	{
+ 		if (*cp == '@')
+ 		{
+ 			*cp++ = 0;
+ 			/* user@host */
+ 			strmcpy(talker->remote_user, names, MAXLOGNAME-1);
+ 			strmcpy(talker->remote_host, cp, MAXHOSTNAMELEN-1);
+ 		} 
+ 		else 
+ 		{
+ 			*cp++ = 0;
+ 			/* host.user or host!user or host:user */
+ 			strmcpy(talker->remote_user, cp, MAXLOGNAME-1);
+ 			strmcpy(talker->remote_host, names, MAXHOSTNAMELEN-1);
+ 		}
+ 	}
+ 	strcpy(talker->remote_tty, empty_string);
+ 
+ 	/* look up the address of the local host */
+ 	if ((hp = gethostbyname(talker->local_host)) == NULL)
+ 		return delete_talk(&talker, "Cannot resolve local hostname");
+ 
+ 	bcopy(hp->h_addr, (char *)&talker->local_addr, hp->h_length);
+ 
+ 	/*
+ 	 * If the callee is on-machine, just copy the
+ 	 * network address, otherwise do a lookup...
+ 	 */
+ 	if (my_stricmp(talker->remote_host, talker->local_host))
+ 	{
+ 		if ((hp = gethostbyname(talker->remote_host)) == NULL)
+ 			return delete_talk(&talker, "Cannot resolve remote hostname");
+ 
+ 		bcopy(hp->h_addr, (char *)&talker->remote_addr, hp->h_length);
+ 	} 
+ 	else
+ 		talker->remote_addr = talker->local_addr;
+ 
+ 	/* find the server's port */
+ 	if ((sp = getservbyname("talk", "udp")) == NULL)
+ 		talker->daemon_port = htons(517);
+ 	else
+ 		talker->daemon_port = sp->s_port;
+ 
+ 	/*
+ 	 * Initialize the message template.
+ 	 */
+ 	talker->msg.vers = TALK_VERSION;
+ 	talker->msg.addr.sa_family = (u_short)htons(AF_INET);
+ 	talker->msg.ctl_addr.sa_family = (u_short)htons(AF_INET);
+ 	talker->msg.id_num = htonl(-1);
+ 	talker->msg.pid = htonl(getpid());
+ 	strmcpy(talker->msg.l_name, talker->local_user, NAME_SIZE-1);
+ 	strmcpy(talker->msg.r_name, talker->remote_user, NAME_SIZE-1);
+ 	strmcpy(talker->msg.r_tty, talker->remote_tty, TTY_SIZE-1);
+ 
+ 	return 0;
+ }
+ 
+ 
+ /* open the ctl socket */
+ static int open_ctl (TalkSession *talker)
+ {
+ 	int length;
+ 
+ 	talker->ctl_addr.sin_addr = talker->local_addr;
+ 	talker->ctl_addr.sin_port = INADDR_ANY;
+ 	talker->ctl_sockt = socket(AF_INET, SOCK_DGRAM, 0);
+ 	if (talker->ctl_sockt <= 0)
+ 		return delete_talk(&talker, "socket() call failed");
+ 
+ 	if (bind(talker->ctl_sockt, (struct sockaddr *)&talker->ctl_addr, sizeof(talker->ctl_addr)) != 0)
+ 		return delete_talk(&talker, "Could not bind local socket");
+ 
+ 	length = sizeof(talker->ctl_addr);
+ 	if (getsockname(talker->ctl_sockt, (struct sockaddr *)&talker->ctl_addr, &length) == -1)
+ 		return delete_talk(&talker, "getsockname() failed");
+ 
+ 	return 0;
+ }
+ 
+ #ifdef __STDC__
+ static int open_sockt(TalkSession *talker)
+ #else
+ static int open_sockt(talker)
+ TalkSession *talker;
+ #endif
+ {
+ 	int length;
+ 
+ 	talker->my_addr.sin_addr = talker->local_addr;
+ 	talker->my_addr.sin_port = INADDR_ANY;
+ 	talker->sockt = socket(AF_INET, SOCK_STREAM, 0);
+ 	if (talker->sockt <= 0)
+ 		return delete_talk(&talker, "socket() call failed");
+ 
+ 	if (bind(talker->sockt, (struct sockaddr *)&talker->my_addr, sizeof(talker->my_addr)) != 0)
+ 		return delete_talk(&talker, "Could not bind local socket");
+ 
+ 	length = sizeof(talker->my_addr);
+ 	if (getsockname(talker->sockt, (struct sockaddr *)&talker->my_addr, &length) == -1)
+ 		return delete_talk(&talker, "getsockname() failed");
+ 
+ 	return 0;
+ }
+ 
+ /*
+  * See if the remote daemon has an invitation for us.
+  */
+ #ifdef __STDC__
+ static int check_for_invitation(TalkSession *talker)
+ #else
+ static int check_for_invitation(talker)
+ TalkSession *talker;
+ #endif
+ {
+ 	CTL_RESPONSE response, *rp = &response;
+ 
+ yell("Checking for invitation");
+ 	/*
+ 	 * set up our side of the 'msg' struct
+ 	 */
+ 	talker->msg.ctl_addr = *(struct sockaddr *)&talker->ctl_addr;
+ 
+ 	/*
+ 	 * ask the remote daemon if we were invited 
+ 	 */
+ 	if (ctl_transact(talker, &talker->remote_addr, LOOK_UP, rp) == -1)
+ 		return -1;
+ 
+ 	if (rp->answer != SUCCESS)
+ 		return 1;		/* an invitation is required */
+ yell("Hrm. looks like we were invited");
+ 
+ 	/*
+ 	 * There was an invitation waiting for us, 
+ 	 * so connect with the other (hopefully waiting) party 
+ 	 */
+ 	talker->msg.id_num = rp->id_num;
+ 	if (rp->addr.sa_family != AF_INET)
+ 		return delete_talk(&talker, "Response uses invalid network address");
+ 
+ 	if (connect(talker->sockt, (struct sockaddr *)&rp->addr, sizeof (rp->addr)) != -1)
+ 	{
+ 		talker->usable = 1;
+ 		yell("TALK (%s): Connection established", talker->user_ref);
+ 		return (0); /* success! */
+ 	}
+ 
+ 	if (errno == ECONNREFUSED) 
+ 	{
+ 		/*
+ 		 * The caller gave up, but his invitation somehow
+ 		 * was not cleared. Clear it and give up.  This should
+ 		 * never happen, or if it does happen, it is because
+ 		 * of a race condition such that i dont think it matters
+ 		 * that we give up here.
+ 		 */
+ 		if (ctl_transact(talker, &talker->remote_addr, DELETE, rp) == -1)
+ 			return -1;
+ 		return delete_talk(&talker, "Remote peer revoked invitation");
+ 	}
+ 	return delete_talk(&talker, "Unable to connect with initiator");
+ }
+ 
+ /*
+  * There wasn't an invitation waiting, so send a request containing
+  * our sockt address to the remote talk daemon so it can invite him 
+  */
+ #ifdef __STDC__
+ static int invite_remote(TalkSession *talker)
+ #else
+ static int invite_remote(talker)
+ TalkSession *talker;
+ #endif
+ {
+ 	if (talker->sockt < 0)
+ 		return delete_talk(&talker, "Invalid fd for listen");
+ 
+ 	if (listen(talker->sockt, 5) != 0)
+ 		return delete_talk(&talker, "listen() call failed");
+ 
+ 	/* 
+ 	 * Tell the remote peer that we've invited them.
+ 	 * the ``send_talk_announce'' function takes care of making
+ 	 * sure that it will be called again, so we dont have to
+ 	 * worry about that here.
+ 	 */
+ 	/* Dont move this next line because ``send_talk_announce''
+ 	 * depends on it being set here!
+ 	 */
+ 	talker->invitation_waiting = 1;
+ 	if (send_talk_announce((void *)talker) == -1)
+ 		return -1;
+ 
+ 	/*
+ 	 * tell my talkd that ive invited the person
+ 	 */
+ 	if (leave_invitation(talker) == -1)
+ 		return -1;
+ 
+ 	/*
+ 	 * All ok so far...
+ 	 */
+ 	talker->status = "Waiting for remote party to respond to invitation";
+ 	say("TALK (%s): Invitation sent -- waiting for them to reply", talker->user_ref);
+ 	return 0;
+ }
+ 
+ 
+ /*
+  * Trade edit characters with the other talk.  Since we are doing
+  * local editing (ie, not "live"), our values dont matter, but theirs do.
+  */
+ #ifdef __STDC__
+ static int set_edit_chars(TalkSession *talker)
+ #else
+ static int set_edit_chars (talker)
+ TalkSession *talker;
+ #endif
+ {
+ 	char buf[3];
+ 	int cc;
+ 	
+ 	buf[0] = talker->local_edit.cerase = '\177';  /* character-erase, ^? */
+ 	buf[1] = talker->local_edit.kill   = '\025';  /* line-erase, ^U */
+ 	buf[2] = talker->local_edit.werase = '\027';  /* word-erase, ^W */
+ 
+ 	cc = write(talker->sockt, buf, 3);
+ 	if ( cc != 3)
+ 		return delete_talk(&talker, "could not write ctl chars");
+ 
+ 	cc = read(talker->sockt, buf, 3);
+ 	if ( cc != 3 )
+ 		return delete_talk(&talker, "could not read ctl chars");
+ 
+ 	talker->remote_edit.cerase = buf[0];
+ 	talker->remote_edit.kill = buf[1];
+ 	talker->remote_edit.werase = buf[2];
+ 
+ 	/*
+ 	 * Its a go!
+ 	 */
+ 	talker->status = "Connection established";
+ 	return 0;
+ }
+ 
+ 
+ 
+ 
+ 
+ 
+ /* * * * * * * * Other functions * * * * * * * * * * */
+ 
+ /*
+  * Send a control command to a remote talk daemon.  This uses UDP,
+  * so we have to be prepared to re-send the command if it doesnt
+  * get there.  For sanity, we place an upper bound of 3 tries before
+  * we give up.  The "type" may be one of:
+  *
+  * LOOKUP	- ask server if we have been invited
+  * ANNOUNCE	- ask that someone else be invited
+  * DELETE	- cancel invitation
+  * LEAVE_INVITE - tell talkd that we want them to remember
+  *		  the invitation (duh)
+  *
+  */
+ #ifdef __STDC__
+ static int ctl_transact(TalkSession *talker, struct in_addr *target, int type, CTL_RESPONSE *rp)
+ #else
+ static int ctl_transact(talker, target, type, rp)
+ TalkSession *talker;
+ struct in_addr *target;
+ int type;
+ CTL_RESPONSE *rp;
+ #endif
+ {
+ 	int nready = 0, cc, i;
+ 	fd_set read_mask;
+ 	struct timeval wait;
+ 
+ 	talker->msg.type = type;
+ 	talker->daemon_addr.sin_addr = *target;
+ 	talker->daemon_addr.sin_port = talker->daemon_port;
+ 
+ 	/*
+ 	 * Keep sending the message until a response of
+ 	 * the proper type is obtained.
+ 	 */
+ 	do 
+ 	{
+ 		wait.tv_sec = 2; /* wait two seconds for a response */
+ 		wait.tv_usec = 0;
+ 		FD_ZERO(&read_mask);
+ 		FD_SET(talker->ctl_sockt, &read_mask);
+ 
+ 		/* resend message until a response is obtained */
+ 		for (i = 1; (i <= 3) && !nready ; i++)
+ 		{
+ 			cc = sendto(talker->ctl_sockt, 
+ 				    (char *)&talker->msg, 
+ 				    sizeof (talker->msg), 
+ 				    0,
+ 				    (struct sockaddr *)&talker->daemon_addr,
+ 				    sizeof(talker->daemon_addr));
+ 
+ 			if (cc != sizeof (talker->msg)) 
+ 				return delete_talk(&talker, "Error on write to talk daemon");
+ 
+ 			if ((nready = select(talker->ctl_sockt + 2, &read_mask, 0, 0, &wait)) < 0)
+ 				return delete_talk(&talker, "Error waiting for daemon response");
+ 		} 
+ 
+ 		if (!nready)
+ 			return delete_talk(&talker, "Looks like there's no talk daemon");
+ 
+ 		/*
+ 		 * Keep reading while there are queued messages 
+ 		 * (this is not necessary, it just saves extra
+ 		 * request/acknowledgements being sent)
+ 		 */
+ 		do 
+ 		{
+ 			FD_SET(talker->ctl_sockt, &read_mask);
+ 
+ 			/* an immediate poll */
+ 			wait.tv_sec = 0;
+ 			nready = select(32, &read_mask, 0, 0, &wait);
+ 			if (nready > 0)
+ 				if ((cc = recv(talker->ctl_sockt, (char *)rp, sizeof (*rp), 0) < 0))
+ 					return delete_talk(&talker, "Error on read from talk daemon");
+ 		} 
+ 		while (nready > 0 && (rp->vers != TALK_VERSION || rp->type != type));
+ 	} 
+ 	while (rp->vers != TALK_VERSION || rp->type != type);
+ 
+ 	rp->id_num = ntohl(rp->id_num);
+ 	rp->addr.sa_family = ntohs(rp->addr.sa_family);
+ 
+ 	return 0;
+ }
+ 
+ 
+ /* This is here because we want it to be hidden. */
+ static TalkSession *TalkList;
+ 
+ /*
+  *
+  * 			read/write from talk sessions
+  *
+  */
+ 
+ #ifdef __STDC__
+ void set_talk_bits (fd_set *rd)
+ #else
+ void set_talk_bits (rd)
+ fd_set *rd;
+ #endif
+ {
+ 	TalkSession *talker;
+ 	for (talker = TalkList; talker; talker = talker->next)
+ 	{
+ 		if (talker->sockt)
+ 			FD_SET(talker->sockt, rd);
+ 	}
+ }
+ 
+ #ifdef __STDC__
+ int check_talks(fd_set *read_set)
+ #else
+ int check_talks(read_set)
+ fd_set *read_set;
+ #endif
+ {
+ 	TalkSession *talker;
+ 	char c, c1[2];
+ 
+ 	for (talker = TalkList; talker; talker = talker->next)
+ 	{
+ 		if (FD_ISSET(talker->sockt, read_set))
+ 		{
+ 			if (talker->usable)
+ 			{
+ 				switch (read(talker->sockt, &c, 1))
+ 				{
+ 					case -1:
+ 						return delete_talk(&talker, (char *)sys_errlist[errno]);
+ 					case 0:
+ 						return delete_talk(&talker, "EOF -- remote peer closed connection");
+ 				}
+ 
+ 				     if (c == talker->remote_edit.kill)
+ 					new_free(&talker->this_line);
+ 				else if (c == talker->remote_edit.cerase)
+ 					chop(talker->this_line);
+ 				else if (c == talker->remote_edit.werase)
+ 					chop_word(talker->this_line);
+ 				else if (c == '\n')
+ 				{
+ 					if (do_hook(TALK_LIST, "%d %s", talker->sockt, talker->this_line))
+ 						put_it("#%s@%s# %s", talker->remote_user, talker->remote_host, talker->this_line);
+ 					new_free(&talker->this_line);
+ 				}
+ 				else
+ 				{
+ 					c1[0] = c; c1[1] = 0;
+ 					malloc_strcat(&talker->this_line, c1);
+ 				}
+ 			}
+ 			else if (talker->invitation_waiting)
+ 				establish_waiting_talk_connection(talker);
+ 			else
+ 				delete_talk(&talker, "talk connection ready and i dont know why.");
+ 		}
+ 	}
+ 	return 0;
+ }
+ 
+ #ifdef __STDC__
+ int write_to_talk (char *userhost, char *line)
+ #else
+ int write_to_talk(userhost, line)
+ char *userhost, *line;
+ #endif
+ {
+ 	TalkSession *talker;
+ 	if ((talker = lookup_talk(userhost)) == NULL)
+ 	{
+ 		yell("No talk open to (%s)", userhost);
+ 		return -1;
+ 	}
+ 
+ 	if (write(talker->sockt, line, strlen(line)) < strlen(line))
+ 		return delete_talk(&talker, "End of File or write() error");
+ 	if (write(talker->sockt, "\n", 1) == -1)
+ 		return delete_talk(&talker, "End of File or write() error");
+ 
+ 	return 0;
+ }
+ 
+ 
+ 
+ /*
+  *
+  *	Local functions to handle creating and disposing of talk sessions
+  *
+  */
+ static TalkSession *get_new_talk _((void))
+ {
+ 	TalkSession *tmptop = TalkList;
+ 	TalkSession *new_talk = (TalkSession *)new_malloc(sizeof(TalkSession));
+ 	bzero(new_talk, sizeof(TalkSession));
+ 
+ #ifdef BSD4_4
+ 	new_talk->daemon_addr.sin_len = sizeof(struct sockaddr_in);
+ 	new_talk->ctl_addr.sin_len = sizeof(struct sockaddr_in);
+ 	new_talk->my_addr.sin_len = sizeof(struct sockaddr_in);
+ #endif
+ 	new_talk->daemon_addr.sin_family = AF_INET;
+ 	new_talk->ctl_addr.sin_family = AF_INET;
+ 	new_talk->my_addr.sin_family = AF_INET;
+ 
+ 	new_talk->local_id = htonl(-1);
+ 	new_talk->remote_id = htonl(-1);
+ 	new_talk->timer_refnum = -1;
+ 	new_talk->closed = 0;
+ 	new_talk->invitation_waiting = 0;
+ 
+ 	TalkList = new_talk;
+ 	new_talk->next = tmptop;
+ 
+ 	return new_talk;
+ }
+ 
+ 
+ 
+ #ifdef __STDC__
+ static int delete_talk (TalkSession **talker, char *message)
+ #else
+ static int delete_talk (talker, message)
+ TalkSession **talker;
+ char *message;
+ #endif
+ {
+ 	TalkSession *talkptr = TalkList;
+ 
+ 	if (message)
+ 		yell("TALK (%s): Connection Closed - %s : %s", (*talker)->user_ref,  message, errno ? sys_errlist[errno] : "");
+ 
+ 	/*
+ 	 * Avoid recursion loops
+ 	 */
+ 	if (!(*talker)->closed)
+ 	{
+ 		if ((*talker)->invitation_waiting)
+ 			send_delete(*talker);
+ 		(*talker)->closed = 1;
+ 	}
+ 
+ 	/*
+ 	 * cancel any pending timers
+ 	 */
+ 	if ((*talker)->timer_refnum != -1)
+ 		delete_timer((*talker)->timer_refnum);
+ 
+ 	/*
+ 	 * close the sockets, warnings are spurious
+ 	 */
+ 	close((*talker)->sockt);
+ 	close((*talker)->ctl_sockt);
+ 
+ 	/* Remote the talk session from the list */
+ 	if (talkptr == *talker)
+ 		TalkList = (*talker)->next;
+ 
+ 	while (talkptr && talkptr->next && talkptr->next != *talker)
+ 		talkptr = talkptr->next;
+ 
+ 	if (talkptr && talkptr->next)
+ 		talkptr->next = talkptr->next->next;
+ 	else
+ 		TalkList = (TalkSession *) 0;
+ 
+ 	new_free((char **)talker);
+ 	return -1;
+ }
+ 
+ /*
+  * Tell the daemon to remove your invitation
+  */
+ #ifdef __STDC__
+ static int send_delete(TalkSession *talker)
+ #else
+ static int send_delete (talker)
+ TalkSession *talker;
+ #endif
+ {
+ 	talker->msg.type = DELETE;
+ 
+ 	/*
+ 	 * This is just a extra clean up, so just send it
+ 	 * and don't wait for an answer
+ 	 */
+ 	talker->msg.id_num = htonl(talker->remote_id);
+ 	talker->daemon_addr.sin_addr = talker->remote_addr;
+ 	if (sendto(talker->ctl_sockt, (char *)&talker->msg, sizeof (talker->msg), 0,
+ 	    (struct sockaddr *)&talker->daemon_addr,
+ 	    sizeof (talker->daemon_addr)) != sizeof(talker->msg))
+ 	{
+ 		return delete_talk(&talker, "send_delete (remote) failed");
+ 	}
+ 	talker->msg.id_num = htonl(talker->local_id);
+ 	talker->daemon_addr.sin_addr = talker->local_addr;
+ 	if (sendto(talker->ctl_sockt, (char *)&talker->msg, sizeof (talker->msg), 0,
+ 	    (struct sockaddr *)&talker->daemon_addr,
+ 	    sizeof (talker->daemon_addr)) != sizeof (talker->msg))
+ 	{
+ 		return delete_talk(&talker, "send_delete (local) failed");
+ 	}
+ 
+ 	return 0;
+ }
+ 
+ /*
+  * both "leave_invitation" and "send_talk_announce" use this function
+  * to set up the local size of the 'msg' struct before sending a
+  * control request.
+  */
+ #ifdef __STDC__
+ static void setup_local_msg (TalkSession *talker, long id)
+ #else
+ static void setup_local_msg (talker, id)
+ TalkSession *talker;
+ long id;
+ #endif
+ {
+ 	/* 
+ 	 * Fill out our side of the 'msg' struct 
+ 	 */
+ 	/* copy new style sockaddr to old, swap family (short in old) */
+ 	talker->msg.addr = *(struct sockaddr *)&talker->my_addr;
+ 
+ /*
+ 	talker->msg.id_num = htonl(id);
+ */
+ 	talker->msg.id_num = htonl(-1);
+ }
+ 
+ #ifdef __STDC__
+ static int leave_invitation (TalkSession *talker)
+ #else
+ static int leave_invitation (talker)
+ TalkSession *talker;
+ #endif
+ {
+ 	CTL_RESPONSE response;
+ 
+ yell("Leaving invitation for %s...", talker->msg.r_name);
+ 	setup_local_msg(talker, talker->local_id);
+ 	if (ctl_transact(talker, &talker->local_addr, LEAVE_INVITE, &response) == -1)
+ 		return -1;
+ 	talker->local_id = response.id_num;
+         if (response.answer != SUCCESS) 
+         {
+ yell("error #3");
+                 if (response.answer < NANSWERS)
+                         delete_talk(&talker, answers[response.answer]);
+                 else
+                         delete_talk(&talker, "Unknown error");
+ 
+                 return -1;
+         }
+ 
+ 	return 0;
+ }
+ 
+ #ifdef __STDC__
+ static int send_talk_announce (void *arg)
+ #else
+ static int send_talk_announce (arg)
+ char *arg;
+ #endif
+ {
+ 	CTL_RESPONSE response;
+ 	TalkSession *talker = (TalkSession *)arg;
+ 
+ 	/* we've established a connection, so we just die */
+ 	if (!talker->invitation_waiting)
+ 		return 0;
+ 	talker->timer_refnum = -1;
+ 
+ yell("Leaving an announcement for %s", talker->msg.r_name);
+ 	setup_local_msg(talker, talker->remote_id);
+ 	if (ctl_transact(talker, &talker->remote_addr, ANNOUNCE, &response) == -1)
+ 		return -1;
+ 	talker->remote_id = response.id_num;
+         if (response.answer != SUCCESS) 
+         {
+ yell("Talk error #4");
+                 if (response.answer < NANSWERS)
+                         delete_talk(&talker, answers[response.answer]);
+                 else
+                         delete_talk(&talker, "Unknown error");
+ 
+                 return -1;
+         }
+ 	talker->timer_refnum = add_timer(-1, 30, send_talk_announce, (char *)talker, NULL);
+ 	return 0;
+ }
+ 
+ #ifdef __STDC__
+ static int establish_waiting_talk_connection (TalkSession *talker)
+ #else
+ static int establish_waiting_talk_connection (talker)
+ TalkSession *talker;
+ #endif
+ {
+ 	int new_sockt;
+ 
+ 	/*
+ 	 * This shouldnt block, because the caller is supposed
+ 	 * to check for that before it calls us
+ 	 */
+ 	if ((new_sockt = accept(talker->sockt, 0, 0)) < 0) 
+ 		return delete_talk(&talker, "Unable to connect with your party");
+ 
+ 	talker->invitation_waiting = 0;
+ 	talker->usable = 1;
+ 	if (talker->timer_refnum != -1)
+ 		delete_timer(talker->timer_refnum);
+ 	talker->timer_refnum = -1;
+ 	yell("TALK (%s): Connection established", talker->user_ref);
+ 	
+ 	/* 
+ 	 * Close the listening socket, save the new socket 
+ 	 */
+ 	close(talker->sockt);
+ 	talker->sockt = new_sockt;
+ 
+ 	if (set_edit_chars(talker) == -1)
+ 		return -1;
+ 
+ 	/*
+  	 * Cancel all pending invitiations
+ 	 */
+ 	return send_delete(talker);
+ }
+ 
+ 
+ #ifdef __STDC__
+ static TalkSession *lookup_talk(char *userhost)
+ #else
+ static TalkSession *lookup_talk(userhost)
+ char *userhost;
+ #endif
+ {
+ 	TalkSession *talker;
+ 
+ 	for (talker = TalkList; talker; talker = talker->next)
+ 	{
+ 		if (!my_stricmp(userhost, talker->user_ref))
+ 			return talker;
+ 	}
+ 
+ 	return NULL;
+ }
+ 
+ static void list_talk _((void))
+ {
+ 	TalkSession *talker;
+ 	char *format = "%-20s %-12s %-20s %-20s";
+ 
+ 	if (!TalkList)
+ 	{
+ 		say("No talk sessions open");
+ 		return;
+ 	}
+ 
+ 	yell(format, "Reference", "User", "Host", "Status");
+ 	for (talker = TalkList; talker; talker = talker->next)
+ 	{
+ 		yell(format, talker->user_ref, talker->remote_user,
+ 			talker->remote_host, talker->status);
+ 	}
+ }
+ 
+ #endif /* TALK_SUPPORT */
